@@ -12,24 +12,22 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+extern crate env_logger;
 extern crate hyper;
 extern crate openstack;
 
 use std::env;
 
-use openstack::auth::Identity;
-#[cfg(feature = "compute")]
-use openstack::compute;
-use openstack::Session;
-
 
 #[cfg(feature = "compute")]
 fn main() {
-    let identity = Identity::from_env()
-        .expect("Failed to create an identity provider from the environment");
-    let session = Session::new(identity);
+    env_logger::init().unwrap();
 
-    let manager = compute::servers::manager(&session);
+    let identity = openstack::auth::Identity::from_env()
+        .expect("Failed to create an identity provider from the environment");
+    let session = openstack::Session::new(identity);
+
+    let manager = openstack::compute::v2(&session).servers();
     let server = manager.get(env::args().nth(1).expect("Provide a server ID"))
         .expect("Cannot get a server");
     println!("ID = {}, Name = {}, Status = {}",
