@@ -12,23 +12,20 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+extern crate env_logger;
 extern crate hyper;
 extern crate openstack;
 
-use openstack::auth::Identity;
-#[cfg(feature = "compute")]
-use openstack::compute;
-use openstack::Session;
-
 
 #[cfg(feature = "compute")]
 fn main() {
-    let identity = Identity::from_env()
-        .expect("Failed to create an identity provider from the environment");
-    let session = Session::new(identity);
+    env_logger::init().unwrap();
 
-    let manager = compute::servers::manager(&session);
-    let servers = manager.list().expect("Cannot list servers");
+    let identity = openstack::auth::Identity::from_env()
+        .expect("Failed to create an identity provider from the environment");
+    let session = openstack::Session::new(identity);
+    let servers = openstack::compute::v2(&session).servers().list()
+        .expect("Cannot list servers");
     for s in &servers {
         println!("ID = {}, Name = {}", s.id(), s.name());
     }
