@@ -73,7 +73,6 @@ extern crate serde;
 extern crate serde_derive;
 extern crate serde_json;
 extern crate time;
-extern crate uuid;
 
 #[cfg(test)] #[macro_use]
 extern crate yup_hyper_mock;
@@ -91,4 +90,5 @@ mod utils;
 pub use common::ApiError;
 pub use common::ApiResult;
 pub use common::ApiVersion;
+pub use common::ApiVersionRequest;
 pub use session::Session;
