@@ -13,50 +13,92 @@
 // limitations under the License.
 
 //! Session structure definition.
-//!
-//! The Session object serves as a wrapper around an HTTP(s) client, handling
-//! authentication, accessing the service catalog and token refresh.
+
+use std::cell::Ref;
+use std::collections::HashMap;
 
 use hyper::{Client, Url};
 use hyper::client::IntoUrl;
 use hyper::method::Method;
 
-use super::ApiResult;
+use super::{ApiError, ApiResult, ApiVersion, ApiVersionRequest};
 use super::auth::Method as AuthMethod;
 use super::http::AuthenticatedRequestBuilder;
+use super::service::{ServiceInfo, ServiceType};
 use super::utils;
 
 
 /// An OpenStack API session.
 ///
-/// Owns a token and an underlying client.
+/// The session object serves as a wrapper around an HTTP(s) client, handling
+/// authentication, accessing the service catalog and token refresh.
+///
+/// The session object also owns region and endpoint interface to use.
+///
+/// Finally, the session object is responsible for API version negotiation.
 #[derive(Debug)]
 pub struct Session<Auth: AuthMethod> {
     auth: Auth,
     client: Client,
     cached_token: utils::ValueCache<Auth::TokenType>,
-    default_region: Option<String>
+    cached_info: utils::MapCache<(&'static str, String), ServiceInfo>,
+    api_versions: HashMap<&'static str, ApiVersion>,
+    region: Option<String>,
+    endpoint_interface: String
 }
 
 
 impl<'a, Auth: AuthMethod + 'a> Session<Auth> {
     /// Create a new session with a given authentication plugin.
+    ///
+    /// The resulting session will use the default endpoint interface (usually,
+    /// public) and the first available region.
     pub fn new(auth_method: Auth) -> Session<Auth> {
+        let ep = auth_method.default_endpoint_interface();
+        let region = auth_method.default_region();
         Session {
             auth: auth_method,
             client: utils::http_client(),
             cached_token: utils::ValueCache::new(None),
-            default_region: None
+            cached_info: utils::MapCache::new(),
+            api_versions: HashMap::new(),
+            region: region,
+            endpoint_interface: ep
         }
     }
 
-    /// Create a new session with a given authentication plugin and region.
-    pub fn new_with_region(auth_method: Auth, region: String) -> Session<Auth> {
+    /// Convert this session into one using the given region.
+    ///
+    /// Negotiated API versions are reset to their default values.
+    pub fn with_region<S: Into<String>>(self, region: S) -> Session<Auth> {
         Session {
-            auth: auth_method,
-            client: utils::http_client(),
-            cached_token: utils::ValueCache::new(None),
-            default_region: Some(region)
+            auth: self.auth,
+            client: self.client,
+            cached_token: self.cached_token,
+            // ServiceInfo has to be refreshed
+            cached_info: utils::MapCache::new(),
+            // Different regions potentially have different API versions?
+            api_versions: HashMap::new(),
+            region: Some(region.into()),
+            endpoint_interface: self.endpoint_interface
+        }
+    }
+
+    /// Convert this session into one using the given endpoint interface.
+    ///
+    /// Negotiated API versions are kept in the new object.
+    pub fn with_endpoint_interface<S: Into<String>>(self,
+                                                    endpoint_interface: S)
+            -> Session<Auth> {
+        Session {
+            auth: self.auth,
+            client: self.client,
+            cached_token: self.cached_token,
+            // ServiceInfo has to be refreshed
+            cached_info: utils::MapCache::new(),
+            api_versions: self.api_versions,
+            region: self.region,
+            endpoint_interface: endpoint_interface.into()
         }
     }
 
@@ -71,45 +113,108 @@ impl<'a, Auth: AuthMethod + 'a> Session<Auth> {
         &self.auth
     }
 
-    /// Get a default (usually public) endpoint URL.
-    #[inline]
-    pub fn get_default_endpoint<S1: Into<String>>(&self, service_type: S1)
-            -> ApiResult<Url> {
-        self.auth.get_endpoint(service_type.into(),
-                                      None,
-                                      self.default_region.clone(),
-                                      &self)
-    }
-
-    /// Get an endpoint URL.
-    pub fn get_endpoint<S1, S2>(&self, service_type: S1,
-                                endpoint_interface: S2) -> ApiResult<Url>
-            where S1: Into<String>, S2: Into<String> {
-        self.auth.get_endpoint(service_type.into(),
-                                      Some(endpoint_interface.into()),
-                                      self.default_region.clone(),
-                                      &self)
-    }
-
     /// A wrapper for HTTP request.
-    pub fn request<U: IntoUrl>(&'a self, method: Method, url: U)
+    pub fn raw_request<U: IntoUrl>(&'a self, method: Method, url: U)
             -> AuthenticatedRequestBuilder<'a, Auth> {
         AuthenticatedRequestBuilder::new(self.client.request(method, url),
                                          self)
     }
 
+    /// Get service info for the given service.
+    pub fn get_service_info<Srv>(&self) -> ApiResult<ServiceInfo>
+            where Srv: ServiceType {
+        let info = try!(self.get_service_info_ref::<Srv>());
+        Ok(info.clone())
+    }
+
+    /// Negotiate an API version with the service.
+    ///
+    /// Negotiation is based on version information returned from the root
+    /// endpoint. If no minimum version is returned, the current version is
+    /// assumed to be the only supported version.
+    ///
+    /// The resulting API version is cached for this session.
+    pub fn negotiate_api_version<Srv>(&mut self, requested: ApiVersionRequest)
+            -> ApiResult<ApiVersion> where Srv: ServiceType {
+        let key = try!(self.ensure_service_info::<Srv>());
+        let info = self.cached_info.get_ref(&key).unwrap();
+
+        match info.pick_api_version(requested.clone()) {
+            Some(ver) => {
+                let _ = self.api_versions.insert(Srv::catalog_type(), ver);
+                info!("Negotiated API version {} for {} API",
+                      ver, Srv::catalog_type());
+                Ok(ver)
+            },
+            None => {
+                let error = ApiError::UnsupportedApiVersion {
+                    requested: requested,
+                    minimum: info.minimum_version.clone(),
+                    maximum: info.current_version.clone()
+                };
+                warn!("API negotiation failed for {} API: {}",
+                      Srv::catalog_type(), error);
+                Err(error)
+            }
+        }
+    }
+
     fn refresh_token(&self) -> ApiResult<()> {
         self.cached_token.ensure_value(|| {
             self.auth.get_token(&self.client)
         })
     }
+
+    fn ensure_service_info<Srv>(&self) -> ApiResult<(&'static str, String)>
+            where Srv: ServiceType {
+        let key = (Srv::catalog_type(), self.endpoint_interface.clone());
+
+        try!(self.cached_info.ensure_value(key.clone(), |_| {
+            self.get_catalog_endpoint(Srv::catalog_type())
+                .and_then(|ep| Srv::service_info(ep, self))
+        }));
+
+        Ok(key)
+    }
+
+    fn get_catalog_endpoint<S>(&self, service_type: S) -> ApiResult<Url>
+            where S: Into<String> {
+        self.auth.get_endpoint(service_type.into(),
+                               Some(self.endpoint_interface.clone()),
+                               self.region.clone(),
+                               &self)
+    }
+
+    fn get_service_info_ref<Srv>(&self) -> ApiResult<Ref<ServiceInfo>>
+            where Srv: ServiceType {
+        let key = try!(self.ensure_service_info::<Srv>());
+        Ok(self.cached_info.get_ref(&key).unwrap())
+    }
 }
 
+
+impl<Auth: AuthMethod + Clone> Clone for Session<Auth> {
+    fn clone(&self) -> Session<Auth> {
+        Session {
+            auth: self.auth.clone(),
+            // NOTE: hyper::Client does not support Clone
+            client: utils::http_client(),
+            cached_token: self.cached_token.clone(),
+            cached_info: self.cached_info.clone(),
+            api_versions: self.api_versions.clone(),
+            region: self.region.clone(),
+            endpoint_interface: self.endpoint_interface.clone()
+        }
+    }
+}
+
+
 #[cfg(test)]
 pub mod test {
     #![allow(missing_debug_implementations)]
     #![allow(unused_results)]
 
+    use std::collections::HashMap;
     use std::io::Read;
 
     use hyper;
@@ -118,7 +223,7 @@ pub mod test {
 
     use super::super::ApiError;
     use super::super::auth::{Identity, Method, Token, NoAuth, SimpleToken};
-    use super::super::auth::identity::IdentityAuthMethod;
+    use super::super::auth::identity::PasswordAuth;
     use super::super::utils;
     use super::Session;
 
@@ -126,11 +231,15 @@ pub mod test {
                                          token: Auth::TokenType,
                                          region: Option<&str>)
             -> Session<Auth> {
+        let ep = auth.default_endpoint_interface();
         Session {
             auth: auth,
             client: cli,
             cached_token: utils::ValueCache::new(Some(token)),
-            default_region: region.map(From::from)
+            cached_info: utils::MapCache::new(),
+            api_versions: HashMap::new(),
+            region: region.map(From::from),
+            endpoint_interface: ep
         }
     }
 
@@ -140,19 +249,22 @@ pub mod test {
                         utils::http_client(), token, None)
     }
 
-    fn session_with_identity(region: Option<&str>)
-            -> Session<IdentityAuthMethod> {
+    fn session_with_identity() -> Session<PasswordAuth> {
         let id = Identity::new("http://127.0.2.1").unwrap()
             .with_user("user", "pa$$w0rd", "example.com")
             .with_project_scope("cool project", "example.com")
             .create().unwrap();
         let cli = hyper::Client::with_connector(MockCatalog::default());
         let token = SimpleToken(String::from("abcdef"));
+        let ep = id.default_endpoint_interface();
         Session {
             auth: id,
             client: cli,
             cached_token: utils::ValueCache::new(Some(token)),
-            default_region: region.map(From::from)
+            cached_info: utils::MapCache::new(),
+            api_versions: HashMap::new(),
+            region: None,
+            endpoint_interface: ep
         }
     }
 
@@ -220,16 +332,12 @@ pub mod test {
     }
 
     #[test]
-    fn test_session_request() {
+    fn test_session_raw_request() {
         let cli = hyper::Client::with_connector(MockHttp::default());
-        let s = Session {
-            auth: NoAuth::new("http://127.0.0.1/").unwrap(),
-            client: cli,
-            cached_token: utils::ValueCache::new(None),
-            default_region: None
-        };
+        let s = new_with_params(NoAuth::new("http://127.0.0.1/").unwrap(),
+                                cli, SimpleToken(String::from("token")), None);
 
-        let mut resp = s.request(hyper::Post, "http://127.0.0.1/")
+        let mut resp = s.raw_request(hyper::Post, "http://127.0.0.1/")
             .body("body").header(ContentLength(4u64)).send().unwrap();
 
         let mut s = String::new();
@@ -238,16 +346,12 @@ pub mod test {
     }
 
     #[test]
-    fn test_session_request_error() {
+    fn test_session_raw_request_error() {
         let cli = hyper::Client::with_connector(MockHttp::default());
-        let s = Session {
-            auth: NoAuth::new("http://127.0.0.2/").unwrap(),
-            client: cli,
-            cached_token: utils::ValueCache::new(None),
-            default_region: None
-        };
+        let s = new_with_params(NoAuth::new("http://127.0.0.2/").unwrap(),
+                                cli, SimpleToken(String::from("token")), None);
 
-        let err = s.request(hyper::Post, "http://127.0.0.2/")
+        let err = s.raw_request(hyper::Post, "http://127.0.0.2/")
             .body("body").header(ContentLength(4u64)).send().err().unwrap();
 
         match err {
@@ -257,16 +361,12 @@ pub mod test {
     }
 
     #[test]
-    fn test_session_request_unchecked_error() {
+    fn test_session_raw_request_unchecked_error() {
         let cli = hyper::Client::with_connector(MockHttp::default());
-        let s = Session {
-            auth: NoAuth::new("http://127.0.0.2/").unwrap(),
-            client: cli,
-            cached_token: utils::ValueCache::new(None),
-            default_region: None
-        };
+        let s = new_with_params(NoAuth::new("http://127.0.0.2/").unwrap(),
+                                cli, SimpleToken(String::from("token")), None);
 
-        let mut resp = s.request(hyper::Post, "http://127.0.0.2/")
+        let mut resp = s.raw_request(hyper::Post, "http://127.0.0.2/")
             .body("body").header(ContentLength(4u64)).send_unchecked()
             .unwrap();
 
@@ -278,21 +378,26 @@ pub mod test {
     }
 
     #[test]
-    fn test_session_get_endpoint_no_region() {
-        let session = session_with_identity(None);
+    fn test_session_get_catalog_endpoint() {
+        let session = session_with_identity();
 
-        let e1 = session.get_default_endpoint("identity").unwrap();
+        let e1 = session.get_catalog_endpoint("identity").unwrap();
         assert_eq!(&e1.to_string(), "http://localhost:5000/");
-        let e2 = session.get_endpoint("identity", "admin").unwrap();
+
+        match session.get_catalog_endpoint("foo").err().unwrap() {
+            ApiError::EndpointNotFound(ref endp) => assert_eq!(endp, "foo"),
+            other => panic!("Unexpected {}", other)
+        };
+    }
+
+    #[test]
+    fn test_session_get_catalog_endpoint_with_endpoint_interface() {
+        let sess1 = session_with_identity().with_endpoint_interface("admin");
+        let e2 = sess1.get_catalog_endpoint("identity").unwrap();
         assert_eq!(&e2.to_string(), "http://localhost:35357/");
 
-        match session.get_default_endpoint("foo").err().unwrap() {
-            ApiError::EndpointNotFound(ref endp) =>
-                assert_eq!(endp, "foo"),
-            other => panic!("Unexpected {}", other)
-        };
-
-        match session.get_endpoint("identity", "unknown").err().unwrap() {
+        let sess2 = session_with_identity().with_endpoint_interface("unknown");
+        match sess2.get_catalog_endpoint("identity").err().unwrap() {
             ApiError::EndpointNotFound(ref endp) =>
                 assert_eq!(endp, "identity"),
             other => panic!("Unexpected {}", other)
@@ -300,27 +405,24 @@ pub mod test {
     }
 
     #[test]
-    fn test_session_get_endpoint_with_region() {
-        let session = session_with_identity(Some("RegionOne"));
+    fn test_session_get_catalog_endpoint_with_region() {
+        let sess1 = session_with_identity().with_region("RegionOne");
+        let e1 = sess1.get_catalog_endpoint("identity").unwrap();
+        assert_eq!(&e1.to_string(), "http://localhost:5000/");
 
-        let e1 = session.get_endpoint("identity", "admin").unwrap();
+        let sess2 = session_with_identity().with_region("unknown");
+        match sess2.get_catalog_endpoint("identity").err().unwrap() {
+            ApiError::EndpointNotFound(ref endp) =>
+                assert_eq!(endp, "identity"),
+            other => panic!("Unexpected {}", other)
+        };
+    }
+
+    #[test]
+    fn test_session_get_catalog_endpoint_with_region_and_endpoint_interface() {
+        let session = session_with_identity().with_region("RegionOne")
+            .with_endpoint_interface("admin");
+        let e1 = session.get_catalog_endpoint("identity").unwrap();
         assert_eq!(&e1.to_string(), "http://localhost:35357/");
     }
-
-    #[test]
-    fn test_session_get_endpoint_with_region_fail() {
-        let session = session_with_identity(Some("unknown"));
-
-        match session.get_default_endpoint("identity").err().unwrap() {
-            ApiError::EndpointNotFound(ref endp) =>
-                assert_eq!(endp, "identity"),
-            other => panic!("Unexpected {}", other)
-        };
-
-        match session.get_endpoint("identity", "public").err().unwrap() {
-            ApiError::EndpointNotFound(ref endp) =>
-                assert_eq!(endp, "identity"),
-            other => panic!("Unexpected {}", other)
-        };
-    }
 }
